Scriptname CARIM_DetectionScript extends ActiveMagicEffect

Actor Property PlayerRef auto
MagicEffect Property CARIM_MGEF_MovespeedDebuff auto
Spell Property CARIM_Spell_MovespeedDebuffAim auto
Spell Property CARIM_Spell_MovespeedDebuffCast auto
Spell Property CARIM_Spell_MovespeedDebuffReload auto

GlobalVariable Property CARIM_DebuffAiming auto
GlobalVariable Property CARIM_DebuffCasting auto
GlobalVariable Property CARIM_DebuffReloading auto

Event OnEffectStart(Actor akTarget, Actor akCaster)
	AnimRegistrations(akTarget)
	if akTarget == PlayerRef
		GoToState("PlayerState")
	endif
EndEvent

Event OnRaceSwitchComplete()
	Utility.Wait(3.0)
	AnimRegistrations(GetTargetActor())
EndEvent

State PlayerState
Event OnAnimationEventUnregistered(ObjectReference akSource, string asEventName)
	Utility.Wait(1.0)
	AnimRegistrations(PlayerRef)								;might throw error to log if this mgef has been removed from player (game reload). MEH
EndEvent
EndState

Function AnimRegistrations(Actor akTarget)
	RegisterForAnimationEvent(akTarget, "BeginCastRight")
	RegisterForAnimationEvent(akTarget, "BeginCastLeft")
	RegisterForAnimationEvent(akTarget, "CastStop")

	RegisterForAnimationEvent(akTarget, "MRh_SpellFire_Event")
	RegisterForAnimationEvent(akTarget, "MLh_SpellFire_Event")

	RegisterForAnimationEvent(akTarget, "bowDraw")
	RegisterForAnimationEvent(akTarget, "bowRelease")

	RegisterForAnimationEvent(akTarget, "arrowRelease")			;reloadStart and reloadStop never seem to get sent for crossbows so these are used instead
	RegisterForAnimationEvent(akTarget, "attackStop")			;attackStop is working well, but may have to register all interrupt anims: staggerStop, recoilStop, GetUpBegin, etc

	;fail-safe events if aiming/reloading anims/sequence get interrupted for any reason
	RegisterForAnimationEvent(akTarget, "weaponSheathe")		;DO NOT use weaponDraw because it can get sent even when the other hand is casting
	RegisterForAnimationEvent(akTarget, "InterruptCast")
EndFunction

Event OnAnimationEvent(ObjectReference akSource, string asEventName)
	Actor akActor = akSource as Actor
	Int iWeaponTypeL = akActor.GetEquippedItemType(0)
	Int iWeaponType = akActor.GetEquippedItemType(1)
	bool bAdjust = False
	if iWeaponTypeL == 7 || iWeaponTypeL == 12 || iWeaponTypeL == 8 || iWeaponTypeL == 9 || iWeaponType == 8 || iWeaponType == 9
		akActor.ModActorValue("CarryWeight", 0.01)				;alleviates issue where animations don't adjust to new movespeed if you begin those animations already in motion (1st-person only, 3rd-Person uses different fix)
		bAdjust = True
	endif
	if iWeaponTypeL == 7 || iWeaponTypeL == 12
		if iWeaponTypeL == 7 && CARIM_DebuffAiming.GetValueInt()
			if asEventName == "bowDraw" && !akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
				CARIM_Spell_MovespeedDebuffAim.Cast(akSource)
				ArrestMovement(akSource)
			elseif asEventName == "bowRelease"
				akActor.DispelSpell(CARIM_Spell_MovespeedDebuffAim)
			endif
		elseif iWeaponTypeL == 12 && akSource.GetAnimationVariableBool("IsAttacking") && CARIM_DebuffReloading.GetValueInt()
			if asEventName == "arrowRelease" && !akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
				CARIM_Spell_MovespeedDebuffReload.Cast(akSource)
				ArrestMovement(akSource);Game.DisablePlayerControls(true, false, false, false, false, false, false, false)		;not even this works until animation (3rd-person) is completed
			elseif asEventName == "attackStop"
				akActor.DispelSpell(CARIM_Spell_MovespeedDebuffReload)
			endif
		endif
		if asEventName == "attackStop" && akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
			akActor.DispelSpell(CARIM_Spell_MovespeedDebuffAim);removes debuff when de-nocking arrow, or if the aiming/reloading animations/sequence were interrupted or cancelled for any reason
			akActor.DispelSpell(CARIM_Spell_MovespeedDebuffReload)
		endif
	else
		if CARIM_DebuffCasting.GetValueInt()
			if iWeaponType == 9
				if asEventName == "BeginCastRight" && !akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
					CARIM_Spell_MovespeedDebuffCast.Cast(akSource)
					ArrestMovement(akSource)
				elseif asEventName == "MRh_SpellFire_Event"
					if akActor.GetEquippedSpell(1).GetNthEffectMagicEffect(0).GetCastingType() == 1 && (!akActor.GetAnimationVariableBool("IsCastingLeft") || iWeaponTypeL != 9)
						akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
					elseif akActor.GetEquippedSpell(1).GetNthEffectMagicEffect(0).GetCastingType() == 2
						while akActor.GetAnimationVariableBool("IsCastingRight")
							Utility.Wait(0.1)
						endwhile
						if iWeaponTypeL != 9 || (!akActor.GetAnimationVariableBool("IsCastingLeft") && !akActor.GetAnimationVariableBool("IsCastingDual") && iWeaponTypeL == 9)
							akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
						endif
					endif
				endif
			endif
			if iWeaponTypeL == 9
				if asEventName == "BeginCastLeft" && !akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
					CARIM_Spell_MovespeedDebuffCast.Cast(akSource)
					ArrestMovement(akSource)
				elseif asEventName == "MLh_SpellFire_Event"
					if akActor.GetEquippedSpell(0).GetNthEffectMagicEffect(0).GetCastingType() == 1 && (!akActor.GetAnimationVariableBool("IsCastingRight") || iWeaponType != 9 || !akActor.GetAnimationVariableBool("IsCastingDual"))
						akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
					elseif akActor.GetEquippedSpell(0).GetNthEffectMagicEffect(0).GetCastingType() == 2
						while akActor.GetAnimationVariableBool("IsCastingLeft")
							Utility.Wait(0.1)
						endwhile
						if iWeaponType != 9 || (!akActor.GetAnimationVariableBool("IsCastingRight") && !akActor.GetAnimationVariableBool("IsCastingDual") && iWeaponType == 9)
							akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
						endif
					endif
				endif
			endif
			if akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff) && (asEventName == "CastStop" || asEventName == "InterruptCast")
				akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
			endif
		endif
	endif
	if asEventName == "weaponSheathe" && akActor.HasMagicEffect(CARIM_MGEF_MovespeedDebuff)
		akActor.DispelSpell(CARIM_Spell_MovespeedDebuffAim)
		akActor.DispelSpell(CARIM_Spell_MovespeedDebuffCast)
		akActor.DispelSpell(CARIM_Spell_MovespeedDebuffReload)
	endif
	if bAdjust
		akActor.ModActorValue("CarryWeight", -0.01)
	endif
	akActor = None
EndEvent

Function ArrestMovement(ObjectReference akSource)
	Actor akActor = akSource as Actor
	if GetState() == "PlayerState" && Game.GetCameraState() != 0
		Game.SetPlayerAIDriven(PlayerRef.GetPlayerControls())
		;using literals could potentially break any mod features wanting to put the player into an AI-driven state so we use GetPlayerControls() to toggle the existing status quo
		akActor.EnableAI(false)
		Utility.Wait(0.25)
		akActor.EnableAI(true)
		Game.SetPlayerAIDriven(PlayerRef.GetPlayerControls())
	endif
	if GetState() == ""
		akActor.EnableAI(false)									;can use literals here because disabled AI won't be sending animation events
		Utility.Wait(0.1)
		akActor.EnableAI(true)
	endif
	akActor = None
EndFunction